export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("CountBars", InputType.int, 1500);
        this.addInput("P", InputType.int, 6);
        this.addInput("PMA", InputType.int, 21);
        this.addInput("Risk", InputType.int, 0);
        this.addInput("ATR_P", InputType.int, 120);
        this.addInput("Q", InputType.float, 0.7);
        this.addInput("Period_WPR", InputType.int, 12);
        this.addInput("HLine", InputType.int, -38);
        this.addInput("LLine", InputType.int, -62);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            val1 : this.addBuffer(),
            val2 : this.addBuffer(),
            val3 : this.addBuffer(),
            val4 : this.addBuffer(),
            val5 : this.addBuffer(),
            val6 : this.addBuffer(),
        };
    }

    onInit() {
        this.buffers.val1
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.ArrowDownWards)
            .setColor(Color.Red);

        this.buffers.val2
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.ArrowUpWards)

            .setColor(Color.Blue);

        this.buffers.val3
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.Saltire)
            .setColor(Color.DarkOrange);

        this.buffers.val4
            .setShape(Shape.Yellow)
            .setArrow(ArrowSymbols.PointHyphenation)
            .setColor(Color.Red);

        this.buffers.val5
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.Circle)
            .setColor(Color.Magenta);


        this.buffers.val6
            .setShape(Shape.Arrow)
            .setArrow(ArrowSymbols.Circle)
            .setColor(Color.Magenta);
    }

    async onUpdate() {

        const {val1, val2, val3, val4, val5, val6} = this.buffers,
            {CountBars, P, PMA, Risk, ATR_P, Q, Period_WPR, HLine, LLine, PriceType} = this.getInputs(),
            {High, Low, Close} = await Bar.load([Bar.Mode.High, Bar.Mode.Low, Bar.Mode.Close], PriceType),
            barSize = Close.length;

        let countedBars = Bar.counted(),
            HAOpen, i, shift, Flag = 1, LHigh, LLow, ATR, WPR_Filter, Median,
            listMA1, listMA2, listWPR, listATR, drawBegin, localCountBars = CountBars;

        drawBegin = barSize - localCountBars;

        val1.setDrawBegin(drawBegin);
        val2.setDrawBegin(drawBegin);
        val3.setDrawBegin(drawBegin);
        val4.setDrawBegin(drawBegin);
        val5.setDrawBegin(drawBegin);
        val6.setDrawBegin(drawBegin);

        if (localCountBars >= barSize) localCountBars = barSize;

        if (countedBars < 0) return (-1);
        if (countedBars < 1) {
            for (i = 1; i <= localCountBars; i++) val1.set(localCountBars - i, 0);
            for (i = 1; i <= localCountBars; i++) val2.set(localCountBars - i, 0);
            for (i = 1; i <= localCountBars; i++) val3.set(localCountBars - i, 0);
            for (i = 1; i <= localCountBars; i++) val4.set(localCountBars - i, 0);
        }
        listMA1 = await Indicators.MA(null, 0, PMA, 0, Averaging.Simple, AppliedPrice.Median, PriceType);
        listMA2 = await Indicators.MA(null, 0, 1, 0, Averaging.Simple, AppliedPrice.Median, PriceType);
        listWPR = await Indicators.WPR(null, 0, Period_WPR, PriceType);
        listATR = await Indicators.ATR(null, 0, ATR_P, PriceType);

        for (shift = localCountBars - 2; shift >= 0; shift--) {
            HAOpen = listMA1[shift] || 0;
            Median = listMA2[shift] || 0;
            LHigh = High[Bar.highest(High, P, shift + 1)];
            LLow = Low[Bar.lowest(Low, P, shift + 1)];
            WPR_Filter = listWPR[shift] || 0;
            ATR = listATR[shift] || 0;

            if ((High[shift] >= LHigh && Close[shift] > Median && Close[shift] > HAOpen && High[shift] > HAOpen && WPR_Filter >= HLine && Flag < 0) ||
                (Close[shift] >= val3.get(shift + 1) && High[shift] > LHigh && Flag < 0)) {
                val2.set(shift, Math.min(Close[shift], (LLow + 2 * ATR)));
                if (Risk !== 0 && val2.get(shift) - LLow > Risk * Current.Point) val2.set(shift, LLow + Risk * 10 * Current.Point);
                if (Close[shift] < LLow + Risk * 10 * Current.Point) val2.set(shift, Close[shift]);
                Flag = 1;
            }
            if (Flag > 0) {
                val3.set(shift, LLow - (Current.Ask - Current.Bid) * 2);
                val4.set(shift, LHigh + (Current.Ask - Current.Bid) * 3);
            }
            val5.set(shift, val2.get(shift) + (val2.get(shift) - val3.get(shift)) * Q);
            if ((Low[shift] <= LLow && Close[shift] < Median && Close[shift] < HAOpen && Low[shift] < HAOpen && WPR_Filter <= LLine && Flag > 0) ||
                (Close[shift] <= val3.get(shift + 1) && Low[shift] < LLow && Flag > 0)) {
                val1.set(shift, Math.max(Close[shift], (LHigh - 2 * ATR)));
                if (Risk !== 0 && LHigh - val1.get(shift) > Risk * Current.Point) val1.set(shift, LHigh - Risk * 10 * Current.Point);
                if (Close[shift] > LHigh - Risk * 10 * Current.Point) val1.set(shift, Close[shift]);
                Flag = -1;
            }
            if (Flag < 0) {
                val3.set(shift, LHigh + (Current.Ask - Current.Bid) * 3);
                val4.set(shift, LLow - (Current.Ask - Current.Bid) * 2);
            }
            val6.set(shift, val1.get(shift) - (val3.get(shift) - val1.get(shift)) * Q + (Current.Ask - Current.Bid));
        }
    }
}
